@@ -3,6 +3,7 @@ from .epl import EPL
 from .external_shear import ExternalShear
 from .pixelated_convergence import PixelatedConvergence
 from .pixelated_potential import PixelatedPotential
+from .pixelated_deflection import PixelatedDeflection
 from .nfw import NFW
 from .point import Point
 from .pseudo_jaffe import PseudoJaffe
@@ -24,6 +25,7 @@ __all__ = [
     "ExternalShear",
     "PixelatedConvergence",
     "PixelatedPotential",
+    "PixelatedDeflection",
     "Multiplane",
     "NFW",
     "Point",
