@@ -34,7 +34,9 @@ def test(device):
     lens = Multiplane(
         name="multiplane",
         cosmology=cosmology,
-        lenses=[SIE(name=f"sie_{i}", cosmology=cosmology) for i in range(len(xs))],
+        lenses=[
+            SIE(name=f"sie_{i}", cosmology=cosmology, z_s=z_s) for i in range(len(xs))
+        ],
         z_s=z_s,
     )
 
@@ -182,8 +184,10 @@ def test_params(device):
 
     # Test that we can pass a dictionary
     params = {
-        f"plane_{p}": [torch.randn(pixels, pixels, device=device)]
-        for p in range(n_planes)
+        "lenses": {
+            f"plane_{p}": [torch.randn(pixels, pixels, device=device)]
+            for p in range(n_planes)
+        }
     }
 
     kappa_eff = multiplane_lens.effective_convergence_div(x, y, params)
